import pytest
import requests
import responses

from scrapingant_client import (
    ScrapingAntClient,
    ScrapingantInvalidTokenException,
    ScrapingantInvalidInputException,
    ScrapingantInternalException,
    ScrapingantSiteNotReachableException,
    ScrapingantDetectedException,
    ScrapingantTimeoutException,
)
from scrapingant_client.constants import SCRAPINGANT_API_BASE_URL


@responses.activate
def test_invalid_token():
    responses.add(responses.GET, SCRAPINGANT_API_BASE_URL + '/extended',
                  json={'detail': 'wrong token'}, status=403)
    client = ScrapingAntClient(token='invalid_token')
    with pytest.raises(ScrapingantInvalidTokenException):
        client.general_request('example.com')


@responses.activate
def test_invalid_input():
    responses.add(responses.GET, SCRAPINGANT_API_BASE_URL + '/extended',
                  json={'detail': 'wrong url'}, status=422)
    client = ScrapingAntClient(token='some_token')
    with pytest.raises(ScrapingantInvalidInputException) as e:
        client.general_request('bad_url')
    assert 'wrong url' in str(e)


@responses.activate
def test_internal_server_error():
    responses.add(responses.GET, SCRAPINGANT_API_BASE_URL + '/extended',
                  json={}, status=500)
    client = ScrapingAntClient(token='some_token')
    with pytest.raises(ScrapingantInternalException):
        client.general_request('bad_url')


@responses.activate
def test_not_reachable():
    responses.add(responses.GET, SCRAPINGANT_API_BASE_URL + '/extended',
                  json={}, status=404)
    client = ScrapingAntClient(token='some_token')
    with pytest.raises(ScrapingantSiteNotReachableException) as e:
        client.general_request('example.com')
    assert 'The requested URL is not reachable (example.com)' in str(e)


@responses.activate
def test_detected():
    responses.add(responses.GET, SCRAPINGANT_API_BASE_URL + '/extended',
                  json={}, status=423)
    client = ScrapingAntClient(token='some_token')
    with pytest.raises(ScrapingantDetectedException) as e:
        client.general_request('example.com')
    assert 'The anti-bot detection system has detected the request' in str(e)


@responses.activate
def test_timeout():
    responses.add(responses.GET, SCRAPINGANT_API_BASE_URL + '/extended',
                  body=requests.exceptions.ReadTimeout())
    client = ScrapingAntClient(token='some_token')
    with pytest.raises(ScrapingantTimeoutException) as e:
        client.general_request('example.com')
    assert 'Got timeout' in str(e)



from typing import Dict, Optional


def convert_headers(headers: Optional[Dict[str, str]]):
    if headers is None:
        return None
    return {
        f'ant-{header_name}': header_value
        for header_name, header_value in headers.items()
    }



import json

import pytest
import responses
from pytest_httpx import HTTPXMock

from scrapingant_client import ScrapingAntClient, Cookie, ProxyType
from scrapingant_client.constants import SCRAPINGANT_API_BASE_URL


@responses.activate
def test_integration():
    client = ScrapingAntClient(token='test_token')
    responses.add(
        responses.GET,
        url=SCRAPINGANT_API_BASE_URL +
        '/extended'
        '?url=http%3A%2F%2Fexample.com'
        '&cookies=test_name%3Dtest_value'
        '&js_snippet=dGVzdF9qc19zdHJpbmc%3D'
        '&proxy_type=datacenter'
        '&proxy_country=test_country'
        '&wait_for_selector=test_selector'
        '&browser=True',
        json={
            "html": "test_content",
            "cookies": "test_key1=test_value1;test_key2=test_value2",
            "text": "test_text",
            "status_code": 200,
        },
        status=200,
    )
    response = client.general_request(
        url='http://example.com',
        cookies=[Cookie('test_name', 'test_value')],
        headers={
            'testheader': 'test_header_value'},
        js_snippet='test_js_string',
        proxy_type=ProxyType.datacenter,
        proxy_country='test_country',
        wait_for_selector='test_selector',
        browser=True,
    )
    expected = {
        'content': 'test_content',
        'cookies': [Cookie('test_key1', 'test_value1'), Cookie('test_key2', 'test_value2')],
        "text": "test_text",
        'status_code': 200,
    }
    assert response.__dict__ == expected
    assert len(responses.calls) == 1

    headers = responses.calls[0].request.headers
    assert headers['ant-testheader'] == 'test_header_value'
    assert headers['x-api-key'] == 'test_token'


@pytest.mark.asyncio
async def test_integration_async(httpx_mock: HTTPXMock):
    client = ScrapingAntClient(token='test_token')
    httpx_mock.add_response(
        method="GET",
        url=SCRAPINGANT_API_BASE_URL +
        '/extended'
        '?url=http%3A%2F%2Fexample.com'
        '&cookies=test_name%3Dtest_value'
        '&js_snippet=dGVzdF9qc19zdHJpbmc%3D'
        '&proxy_type=datacenter'
        '&proxy_country=test_country'
        '&wait_for_selector=test_selector'
        '&browser=true',
        json={
            "html": "test_content",
            "cookies": "test_key1=test_value1;test_key2=test_value2",
            "text": "test_text",
            "status_code": 200,
        },
        status_code=200,
    )
    response = await client.general_request_async(
        url='http://example.com',
        cookies=[Cookie('test_name', 'test_value')],
        headers={
            'testheader': 'test_header_value'},
        js_snippet='test_js_string',
        proxy_type=ProxyType.datacenter,
        proxy_country='test_country',
        wait_for_selector='test_selector',
        browser=True,
    )
    expected = {
        'content': 'test_content',
        'cookies': [Cookie('test_key1', 'test_value1'), Cookie('test_key2', 'test_value2')],
        "text": "test_text",
        'status_code': 200,
    }
    assert response.__dict__ == expected
    assert len(httpx_mock.get_requests()) == 1

    headers = httpx_mock.get_requests()[0].headers
    assert headers['ant-testheader'] == 'test_header_value'
    assert headers['x-api-key'] == 'test_token'


@responses.activate
def test_post():
    client = ScrapingAntClient(token='test_token')
    responses.add(
        responses.POST,
        url=SCRAPINGANT_API_BASE_URL + '/extended',
        json={
            "html": "test_content",
            "cookies": "",
            "text": "test_text",
            "status_code": 200,
        },
        status=200,
    )
    client.general_request(
        url='http://example.com',
        method='POST',
        json={
            'test_key': 'test_value'},
    )

    assert len(responses.calls) == 1
    assert responses.calls[0].request.method == 'POST'
    assert json.loads(responses.calls[0].request.body) == {
        'test_key': 'test_value'}
    assert responses.calls[0].request.params == {
        'browser': 'True',
        'proxy_type': 'datacenter',
        'url': 'http://example.com',
    }

    headers = responses.calls[0].request.headers
    assert headers['x-api-key'] == 'test_token'



import platform
import sys
from typing import List, Optional, Dict

import requests

import scrapingant_client
from scrapingant_client.constants import SCRAPINGANT_API_BASE_URL, TIMEOUT_SECONDS
from scrapingant_client.cookie import Cookie, cookies_list_to_string, cookies_list_from_string
from scrapingant_client.errors import (
    ScrapingantInvalidTokenException,
    ScrapingantInvalidInputException,
    ScrapingantInternalException,
    ScrapingantSiteNotReachableException,
    ScrapingantDetectedException,
    ScrapingantTimeoutException,
)
from scrapingant_client.headers import convert_headers
from scrapingant_client.proxy_type import ProxyType
from scrapingant_client.response import Response
from scrapingant_client.utils import base64_encode_string


class ScrapingAntClient:
    def __init__(self, token: str):
        self.token = token
        self.requests_session = requests.Session()
        version = scrapingant_client.__version__
        self.user_agent = f'ScrapingAnt Client/{version} ({sys.platform}; Python/{platform.python_version()});'
        self.requests_session.headers.update({
            'x-api-key': self.token,
            'User-Agent': self.user_agent,
        })

    def _form_payload(
            self,
            url: str,
            cookies: Optional[List[Cookie]] = None,
            js_snippet: Optional[str] = None,
            proxy_type: ProxyType = ProxyType.datacenter,
            proxy_country: Optional[str] = None,
            wait_for_selector: Optional[str] = None,
            browser: bool = True,
    ) -> Dict:
        request_data = {'url': url}
        if cookies is not None:
            request_data['cookies'] = cookies_list_to_string(cookies)
        if js_snippet is not None:
            encoded_js_snippet = base64_encode_string(js_snippet)
            request_data['js_snippet'] = encoded_js_snippet
        request_data['proxy_type'] = proxy_type
        if proxy_country is not None:
            request_data['proxy_country'] = proxy_country.lower()
        if wait_for_selector is not None:
            request_data['wait_for_selector'] = wait_for_selector
        request_data['browser'] = browser
        return request_data

    def _parse_response(self, response_status_code: int, response_data: Dict, url: str) -> Response:
        if response_status_code == 403:
            raise ScrapingantInvalidTokenException()
        elif response_status_code == 404:
            raise ScrapingantSiteNotReachableException(url)
        elif response_status_code == 422:
            raise ScrapingantInvalidInputException(response_data)
        elif response_status_code == 423:
            raise ScrapingantDetectedException()
        elif response_status_code == 500:
            raise ScrapingantInternalException()
        content = response_data['html']
        cookies_string = response_data['cookies']
        text = response_data['text']
        status_code = response_data['status_code']
        cookies_list = cookies_list_from_string(cookies_string)
        return Response(
            content=content,
            cookies=cookies_list,
            text=text,
            status_code=status_code
        )

    def general_request(
            self,
            url: str,
            method: str = 'GET',
            cookies: Optional[List[Cookie]] = None,
            headers: Optional[Dict[str, str]] = None,
            js_snippet: Optional[str] = None,
            proxy_type: ProxyType = ProxyType.datacenter,
            proxy_country: Optional[str] = None,
            wait_for_selector: Optional[str] = None,
            browser: bool = True,
            data=None,
            json=None,
    ) -> Response:
        request_data = self._form_payload(
            url=url,
            cookies=cookies,
            js_snippet=js_snippet,
            proxy_type=proxy_type,
            proxy_country=proxy_country,
            wait_for_selector=wait_for_selector,
            browser=browser,
        )
        try:
            response = self.requests_session.request(
                method=method,
                url=SCRAPINGANT_API_BASE_URL + '/extended',
                params=request_data,
                headers=convert_headers(headers),
                data=data,
                json=json,
            )
        except requests.exceptions.Timeout:
            raise ScrapingantTimeoutException()
        response_status_code = response.status_code
        response_data = response.json()
        parsed_response: Response = self._parse_response(response_status_code, response_data, url)
        return parsed_response

    async def general_request_async(
            self,
            url: str,
            method: str = 'GET',
            cookies: Optional[List[Cookie]] = None,
            headers: Optional[Dict[str, str]] = None,
            js_snippet: Optional[str] = None,
            proxy_type: ProxyType = ProxyType.datacenter,
            proxy_country: Optional[str] = None,
            wait_for_selector: Optional[str] = None,
            browser: bool = True,
            data=None,
            json=None,
    ) -> Response:
        import httpx

        request_data = self._form_payload(
            url=url,
            cookies=cookies,
            js_snippet=js_snippet,
            proxy_type=proxy_type,
            proxy_country=proxy_country,
            wait_for_selector=wait_for_selector,
            browser=browser,
        )
        async with httpx.AsyncClient(
                headers={
                    'x-api-key': self.token,
                    'User-Agent': self.user_agent,
                },
                timeout=TIMEOUT_SECONDS,
        ) as client:
            try:
                response = await client.request(
                    method=method,
                    url=SCRAPINGANT_API_BASE_URL + '/extended',
                    params=request_data,
                    headers=convert_headers(headers),
                    data=data,
                    json=json,
                )
            except httpx.TimeoutException:
                raise ScrapingantTimeoutException()
        response_status_code = response.status_code
        response_data = response.json()
        parsed_response: Response = self._parse_response(response_status_code, response_data, url)
        return parsed_response



__version__ = "2.0.0"

from scrapingant_client.client import ScrapingAntClient
from scrapingant_client.cookie import Cookie
from scrapingant_client.errors import (
    ScrapingantClientException,
    ScrapingantInvalidTokenException,
    ScrapingantInvalidInputException,
    ScrapingantInternalException,
    ScrapingantSiteNotReachableException,
    ScrapingantDetectedException,
    ScrapingantTimeoutException,
)
from scrapingant_client.proxy_type import ProxyType
from scrapingant_client.response import Response

__all__ = [
    'ScrapingAntClient',
    'Cookie',
    'ProxyType',
    'ScrapingantClientException',
    'ScrapingantInvalidTokenException',
    'ScrapingantInvalidInputException',
    'ScrapingantInternalException',
    'ScrapingantSiteNotReachableException',
    'ScrapingantDetectedException',
    'ScrapingantTimeoutException',
    'Response',
]



import os
import pathlib
import re

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

HERE = pathlib.Path(__file__).parent
txt = (HERE / "scrapingant_client" / "__init__.py").read_text("utf-8")
version = re.findall(r'^__version__ = "([^"]+)"\r?$', txt, re.M)[0]

setuptools.setup(
    name="scrapingant-client",
    version=version,
    author="andrii.kovalenko",
    author_email="adrekoval@gmail.com",
    license='Apache-2.0',
    description="Official python client for the ScrapingAnt API.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ScrapingAnt/scrapingant-client-python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet :: WWW/HTTP :: Browsers",
        "License :: OSI Approved :: Apache Software License",
    ],
    keywords="scrapingant api scraper scraping",
    python_requires='~=3.7',
    install_requires=['requests>=2,<3'],
    extras_require={
        'dev': [
            'pytest>=7,<8',
            'flake8>=4,<5',
            'responses>=0,<1',
            'pytest-httpx>=0,<1',
            'pytest-asyncio>=0,<1',
        ],
        'async': [
            'httpx<1',
        ],
    },
)
